SystemOrganization addCategory: #TinyBlog!
SystemOrganization addCategory: #'TinyBlog-Core'!
SystemOrganization addCategory: #'TinyBlog-Manifest'!
SystemOrganization addCategory: #'TinyBlog-Teapot'!
SystemOrganization addCategory: #'TinyBlog-Teapot-Tests'!
SystemOrganization addCategory: #'TinyBlog-Tests'!

Object subclass: #TBXBlog
	instanceVariableNames: 'posts'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TinyBlog-Core'!

TBXBlog class
	instanceVariableNames: 'uniqueInstance'!

!TBXBlog commentStamp: 'PeterPesch 6/19/2020 12:29' prior: 0!
I represent a Tiny Blog.

I can show Posts from different Categories.

Peter Pesch, based on the code from the Pharo Mooc.
!

TBXBlog class
	instanceVariableNames: 'uniqueInstance'!

!TBXBlog class methodsFor: 'action' stamp: 'PeterPesch 6/12/2020 15:38'!
createDemoPosts
    "TBBlog createDemoPosts"
    self current
    writeBlogPost: ((TBXPost
        title: 'Welcome in TinyBlog'
        text: 'TinyBlog is a small blog engine made with Pharo.'
        category: 'TinyBlog') visible: true);
    writeBlogPost: ((TBXPost
        title: 'Report Pharo Sprint'
        text: 'Friday, June 12 there was a Pharo sprint / Moose dojo. It was a
nice event with more than 15 motivated sprinters. With the help
of candies, cakes and chocolate, huge work has been done'
        category: 'Pharo') visible: true);
    writeBlogPost: ((TBXPost
        title: 'Brick on top of Bloc - Preview'
        text: 'We are happy to announce the first previewversion of Brick, a new widget set created from scratch on top of Bloc. Brick is being developed primarily by Alex Syrel (together with Alain Plantec, Andrei Chis and myself), and the work is sponsored by ESUG.
Brick is part of the Glamorous Toolkit effort and will provide the basis for the new versions of the development tools.'
        category: 'Pharo') visible: true);
    writeBlogPost: ((TBXPost
        title: 'The sad story of unclassified blog posts'
        text: 'So sad that I can read this.') visible: true);
    writeBlogPost: ((TBXPost
        title: 'Working with Pharo on the Raspberry Pi'
        text: 'Hardware is getting cheaper and many new small devices like the famous Raspberry Pi provide new computation power that was one once only available on regular desktop computers.'
        category: 'Pharo') visible: true)! !

!TBXBlog class methodsFor: 'accessing' stamp: 'PeterPesch 6/12/2020 14:30'!
current
	"Return the singleton instance."
	^ uniqueInstance ifNil: [ uniqueInstance := self new ]! !

!TBXBlog class methodsFor: 'class initialization' stamp: 'PeterPesch 6/18/2020 17:53'!
initialize
	self reset! !

TBXBlog initialize!

!TBXBlog class methodsFor: 'action' stamp: 'PeterPesch 6/12/2020 14:41'!
reset
	uniqueInstance := nil.! !

!TBXBlog methodsFor: 'accessing' stamp: 'PeterPesch 6/12/2020 15:01'!
allBlogPosts
	^ posts
! !

!TBXBlog methodsFor: 'accessing' stamp: 'PeterPesch 6/18/2020 17:50'!
allBlogPostsFromCategory: aCategory
	^ posts select: [ :post | post category = aCategory  ]! !

!TBXBlog methodsFor: 'accessing' stamp: 'PeterPesch 6/12/2020 15:24'!
allCategories
	^ (self allBlogPosts collect: [ :p | p category ]) asSet! !

!TBXBlog methodsFor: 'accessing' stamp: 'PeterPesch 6/19/2020 12:33'!
allVisibleBlogPosts
	^ posts select: #isVisible! !

!TBXBlog methodsFor: 'accessing' stamp: 'PeterPesch 6/12/2020 15:15'!
allVisibleBlogPostsFromCategory: aCategory 
	^ posts select: [ :post | post isVisible and: [ post category = aCategory ] ]! !

!TBXBlog methodsFor: 'initialization' stamp: 'PeterPesch 6/12/2020 14:56'!
initialize
	super initialize.
	posts := OrderedCollection new! !

!TBXBlog methodsFor: 'action' stamp: 'PeterPesch 6/12/2020 14:32'!
removeAllPosts
	posts := OrderedCollection new! !

!TBXBlog methodsFor: 'accessing' stamp: 'PeterPesch 6/12/2020 14:40'!
size
	^ posts size! !

!TBXBlog methodsFor: 'action' stamp: 'PeterPesch 6/12/2020 14:34'!
writeBlogPost: aTBXPost
	"Add the blog post to the blog."
	posts add: aTBXPost ! !

Object subclass: #TBXPost
	instanceVariableNames: 'title text date category visible'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TinyBlog-Core'!

!TBXPost commentStamp: 'PeterPesch 6/19/2020 12:30' prior: 0!
I represent a Post on the Tiny Blog.

I have a tilte, text, date and category, and I may be visible.

Peter Pesch, based on the code from the Pharo Mooc.!

!TBXPost class methodsFor: 'instance creation' stamp: 'PeterPesch 6/10/2020 16:48'!
title: aTitle text: aText
	"Create a new text in the Unclassified category."

	^ self new
		title: aTitle;
		text: aText;
		yourself! !

!TBXPost class methodsFor: 'instance creation' stamp: 'PeterPesch 6/10/2020 16:48'!
title: aTitle text: aText category: aCategory
	"Create a new text in a given category."

	^ (self title: aTitle text: aText)
		category: aCategory;
		yourself! !

!TBXPost class methodsFor: 'classification' stamp: 'PeterPesch 6/10/2020 16:38'!
unclassifiedTag
	"Unclassified Category"
			
	^ 'Unclassified'! !

!TBXPost methodsFor: 'action' stamp: 'PeterPesch 6/10/2020 16:30'!
beVisible
	"Make the post visible"
			
	self visible: true! !

!TBXPost methodsFor: 'acccessing' stamp: 'PeterPesch 6/10/2020 16:23'!
category
	"Return the name of the category of the post"

	^ category! !

!TBXPost methodsFor: 'acccessing' stamp: 'PeterPesch 6/10/2020 16:28'!
category: anObject
	"Updates the name of the category of the post"

	category := anObject ! !

!TBXPost methodsFor: 'acccessing' stamp: 'PeterPesch 6/10/2020 16:22'!
date
	"Return the date of writing"

	^ date! !

!TBXPost methodsFor: 'acccessing' stamp: 'PeterPesch 6/10/2020 16:27'!
date: aDate
	"Updates the date of writing"

	date := aDate! !

!TBXPost methodsFor: 'initialization' stamp: 'PeterPesch 6/10/2020 16:42'!
initialize
	"Intialize ta new post as Unclassified"	

	super initialize.
	self category: self class unclassifiedTag.
	self date: Date today.
	self notVisible! !

!TBXPost methodsFor: 'testing' stamp: 'PeterPesch 6/10/2020 17:16'!
isUnclassified
	"Check whether post is in Unclassified category"	

	^ self category = self class unclassifiedTag! !

!TBXPost methodsFor: 'testing' stamp: 'PeterPesch 6/10/2020 17:14'!
isVisible
	"Check whether post is visible"	

	^ self visible! !

!TBXPost methodsFor: 'action' stamp: 'PeterPesch 6/10/2020 16:31'!
notVisible
	"Hide the post"

	self visible: false! !

!TBXPost methodsFor: 'acccessing' stamp: 'PeterPesch 6/10/2020 16:22'!
text
	"Return the post text"
			
	^ text! !

!TBXPost methodsFor: 'acccessing' stamp: 'PeterPesch 6/10/2020 16:26'!
text: aString
	"Updates the post text"

	text := aString ! !

!TBXPost methodsFor: 'acccessing' stamp: 'PeterPesch 6/10/2020 16:21'!
title
	"Return the post title"
			
	^ title! !

!TBXPost methodsFor: 'acccessing' stamp: 'PeterPesch 6/10/2020 16:25'!
title: aString
	"Updates the post title"

	title := aString ! !

!TBXPost methodsFor: 'acccessing' stamp: 'PeterPesch 6/10/2020 16:24'!
visible
	"Return whether the post is visible or not"

	^ visible! !

!TBXPost methodsFor: 'acccessing' stamp: 'PeterPesch 6/10/2020 16:25'!
visible: aBoolean
	"Updates whether the post is visible or not"

	visible := aBoolean ! !

Object subclass: #TBXTeapotWebApp
	instanceVariableNames: 'teapot'
	classVariableNames: 'Server'
	poolDictionaries: ''
	category: 'TinyBlog-Teapot'!

!TBXTeapotWebApp commentStamp: 'PeterPesch 6/19/2020 12:50' prior: 0!
I represent a simple Web Interface for the Tiny Blog.

I access the TinyBlog API and I use the Teaport Web Platform.

Peter Pesch, based on the code from the Pharo Mooc.!

!TBXTeapotWebApp class methodsFor: 'instance creation' stamp: 'PeterPesch 6/24/2020 14:32'!
singleton
	Server ifNil: [ Server := self new start ].
	^ Server! !

!TBXTeapotWebApp class methodsFor: 'controlling' stamp: 'PeterPesch 6/24/2020 14:16'!
start
    Server ifNil: [ Server := self new start ]! !

!TBXTeapotWebApp class methodsFor: 'controlling' stamp: 'PeterPesch 6/24/2020 14:16'!
stop
    Server ifNotNil: [ :arg | Server stop. Server := nil ]! !

!TBXTeapotWebApp methodsFor: 'contents' stamp: 'PeterPesch 6/19/2020 15:45'!
allPosts
    ^ TBXBlog current allVisibleBlogPosts! !

!TBXTeapotWebApp methodsFor: 'rendering' stamp: 'PeterPesch 6/24/2020 14:42'!
errorPage
	^ String streamContents: [ :s |
		self renderPageHeaderOn: s.
		s << '<p>Oops, an error occurred</p>'.
		self renderPageFooterOn: s ]! !

!TBXTeapotWebApp methodsFor: 'contents' stamp: 'PeterPesch 6/19/2020 15:49'!
homePage
    ^ String streamContents: [ :s |
        self renderPageHeaderOn: s.
        s << '<h1>TinyBlog Web App</h1>'.
        s << '<ul>'.
        self allPosts do: [ :aPost | s << '<li>'; << aPost title; << '</li>'. ].
        s << '</ul>'.
        self renderPageFooterOn: s ]! !

!TBXTeapotWebApp methodsFor: 'initialization' stamp: 'PeterPesch 6/19/2020 12:51'!
initialize
    super initialize.
    teapot := Teapot configure: {
        #port -> 8081.
        #debugMode -> true }! !

!TBXTeapotWebApp methodsFor: 'contents' stamp: 'PeterPesch 6/24/2020 14:45'!
pageForPostNumber: aPostNumber
	| currentPost |
	currentPost := self allPosts at: aPostNumber ifAbsent: [ ^ self errorPage ].
	^ String streamContents: [ :s |
		self renderPageHeaderOn: s.
		s << ('<h1>', currentPost title, '</h1>').
		s << ('<h3>', currentPost date mmddyyyy, '</h3>').
		s << ('<p> Category: ', currentPost category, '</p>').
		s << ('<p>', currentPost text, '</p>').
		self renderPageFooterOn: s ]! !

!TBXTeapotWebApp methodsFor: 'rendering' stamp: 'PeterPesch 6/19/2020 15:48'!
renderPageFooterOn: aStream
    aStream << '</body></html>'! !

!TBXTeapotWebApp methodsFor: 'rendering' stamp: 'PeterPesch 6/19/2020 15:48'!
renderPageHeaderOn: aStream
    aStream << '<html><body>'! !

!TBXTeapotWebApp methodsFor: 'controlling' stamp: 'PeterPesch 6/24/2020 14:44'!
start
    teapot
        GET: '/' -> [ self homePage ];
        GET: '/post/<id>' -> [ :request | self pageForPostNumber: (request at: #id) asNumber ];
        start! !

!TBXTeapotWebApp methodsFor: 'controlling' stamp: 'PeterPesch 6/24/2020 14:17'!
stop
    teapot stop! !

!XMLNode methodsFor: '*TinyBlog' stamp: 'PeterPesch 6/24/2020 12:23'!
allElements: tagname
    "Return all descendant nodes with given tagname"
    | result |
    result := OrderedCollection new.
    self elementName = tagname ifTrue: [ result add: self ].
    self children do: [ :child | result addAll: (child allElements: tagname) ].
    ^ result.! !

!XMLNode methodsFor: '*TinyBlog' stamp: 'PeterPesch 6/23/2020 17:00'!
children
    "Return the children of this XMLNode."
    self hasChildren
        ifTrue: [ ^ self nodes collection ]
        ifFalse: [ ^ OrderedCollection new ]! !

!XMLNode methodsFor: '*TinyBlog' stamp: 'PeterPesch 6/23/2020 16:58'!
elementName 
    "Return the name of this XMLNode, if it is an element."
    self isElement
        ifTrue: [ ^ self name ]
        ifFalse: [ ^ '' ]! !

PackageManifest subclass: #ManifestTinyBlog
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TinyBlog-Manifest'!

!ManifestTinyBlog commentStamp: '<historical>' prior: 0!
I store metadata for this package. These meta data are used by other tools such as the SmalllintManifestChecker and the critics Browser!

!ManifestTinyBlog class methodsFor: 'code-critics' stamp: 'PeterPesch 6/24/2020 14:24'!
ruleGRUsesCurlyBraceArraysRuleV1FalsePositive
	^ #(#(#(#RGMethodDefinition #(#TBXTeapotWebApp #initialize #false)) #'2020-06-24T14:24:30.614032+02:00') )! !

TestCase subclass: #TBXBlogTest
	instanceVariableNames: 'blog post first'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TinyBlog-Tests'!

!TBXBlogTest methodsFor: 'running' stamp: 'PeterPesch 6/12/2020 15:19'!
setUp
	"Initialize the Fixture"
	super setUp.

	blog := TBXBlog current.
	blog removeAllPosts.

	first := TBXPost title: 'A title' text: 'A text' category: 'First Category'.
	blog writeBlogPost: first.

	post := (TBXPost title: 'Another title' text: 'Another text' category: 'Second Category') beVisible! !

!TBXBlogTest methodsFor: 'running' stamp: 'PeterPesch 6/18/2020 17:56'!
tearDown
	TBXBlog reset.
	super tearDown! !

!TBXBlogTest methodsFor: 'tests' stamp: 'PeterPesch 6/12/2020 15:09'!
testAddBlogPost
			
	blog writeBlogPost: post.
	self assert: blog size equals: 2! !

!TBXBlogTest methodsFor: 'tests' stamp: 'PeterPesch 6/12/2020 15:09'!
testAllBlogPosts
	blog writeBlogPost: post.
	self assert: blog allBlogPosts size equals: 2
! !

!TBXBlogTest methodsFor: 'tests' stamp: 'PeterPesch 6/12/2020 15:08'!
testAllBlogPostsFromCategory
	self assert: (blog allBlogPostsFromCategory: 'First Category') size equals: 1
! !

!TBXBlogTest methodsFor: 'tests' stamp: 'PeterPesch 6/12/2020 15:23'!
testAllCategories
	blog writeBlogPost: post.
	self assert: blog allCategories size equals: 2! !

!TBXBlogTest methodsFor: 'tests' stamp: 'PeterPesch 6/19/2020 12:32'!
testAllVisibleBlogPosts
	blog writeBlogPost: post.
	self assert: blog allVisibleBlogPosts size equals: 1
! !

!TBXBlogTest methodsFor: 'tests' stamp: 'PeterPesch 6/12/2020 15:18'!
testAllVisibleBlogPostsFromCategory
	blog writeBlogPost: post.
	self assert: (blog allVisibleBlogPostsFromCategory: 'First Category') size equals: 0.
	self assert: (blog allVisibleBlogPostsFromCategory: 'Second Category') size equals: 1.
! !

!TBXBlogTest methodsFor: 'tests' stamp: 'PeterPesch 6/12/2020 15:22'!
testUnclassifiedBlogPosts
	self assert: (blog allBlogPosts select: [ :p | p isUnclassified ]) size equals: 0! !

TestCase subclass: #TBXPostTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TinyBlog-Tests'!

!TBXPostTest methodsFor: 'tests' stamp: 'PeterPesch 6/10/2020 17:02'!
testPostIsCreatedCorrectly
	"Check whether a new blog post created correctly"

	| post |
	post := TBXPost
		title: 'Welcome to my blog'
		text: 'TinyBlog is a small blog engine made with Pharo.'
		category: 'TinyBlog'.
	self assert: post title equals: 'Welcome to my blog'.
	self assert: post text equals: 'TinyBlog is a small blog engine made with Pharo.'.
	self assert: post category equals: 'TinyBlog'.
! !

!TBXPostTest methodsFor: 'tests' stamp: 'PeterPesch 6/10/2020 17:17'!
testWithoutCategoryIsUnclassified
	"Check whether a new blog post without category is Unclassified"

	| post |
	post := TBXPost
		title: 'Welcome to my blog'
		text: 'TinyBlog is a small blog engine made with Pharo.'.
	self assert: post title equals: 'Welcome to my blog'.
	self assert: post isUnclassified.
	self deny: post isVisible.
! !

TestCase subclass: #TBXTeapotWebAppTest
	instanceVariableNames: 'webAddress'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TinyBlog-Teapot-Tests'!

!TBXTeapotWebAppTest methodsFor: 'running' stamp: 'PeterPesch 6/23/2020 16:51'!
setUp
    "Hooks that subclasses may override to define the fixture of test."
    super setUp.
    webAddress := 'http://localhost:8081/'.! !

!TBXTeapotWebAppTest methodsFor: 'tests' stamp: 'PeterPesch 6/24/2020 14:03'!
testAllPosts
	| result tree node |
	TBXTeapotWebApp stop.
	TBXBlog reset.
	TBXTeapotWebApp start.
	result := ZnEasy get: webAddress.
	tree := (XMLDOMParser on: result entity string) parseDocument.
	node := (tree allElements: 'ul') at: 1.
	self assert: node nodes size equals: 0.
	TBXBlog createDemoPosts.
	result := ZnEasy get: webAddress.
	tree := (XMLDOMParser on: result entity string) parseDocument.
	node := (tree allElements: 'ul') at: 1.
	self assert: node nodes size equals: 5.
	TBXBlog createDemoPosts.
	result := ZnEasy get: webAddress.
	tree := (XMLDOMParser on: result entity string) parseDocument.
	node := (tree allElements: 'ul') at: 1.
	self assert: node nodes size equals: 10.
! !

!TBXTeapotWebAppTest methodsFor: 'tests' stamp: 'PeterPesch 6/24/2020 15:05'!
testErrorPage
	| result tree node |
	TBXTeapotWebApp stop.
	TBXTeapotWebApp start.
	TBXBlog reset.
	TBXBlog createDemoPosts.
	result := ZnEasy get: (webAddress , 'post/9').
	tree := (XMLDOMParser on: result entity string) parseDocument.
	node := (tree allElements: 'p') at: 1.
	node := node nodes at: 1.
	self assert: node string equals: 'Oops, an error occurred'! !

!TBXTeapotWebAppTest methodsFor: 'tests' stamp: 'PeterPesch 6/24/2020 13:54'!
testHomepage
	| result tree node |
	TBXTeapotWebApp stop.
	TBXTeapotWebApp start.
	result := ZnEasy get: webAddress.
	tree := (XMLDOMParser on: result entity string) parseDocument.
	node := (tree allElements: 'h1') at: 1.
	node := node nodes at: 1.
	self assert: node string equals: 'TinyBlog Web App'! !

!TBXTeapotWebAppTest methodsFor: 'tests' stamp: 'PeterPesch 6/23/2020 16:52'!
testStart
    TBXTeapotWebApp stop.
    TBXTeapotWebApp start.
    self shouldnt: (ZnEasy get: webAddress) raise: Error.! !

!TBXTeapotWebAppTest methodsFor: 'tests' stamp: 'PeterPesch 6/24/2020 12:30'!
testStop
    | timeOut |
    TBXTeapotWebApp start.
    TBXTeapotWebApp stop.
    timeOut := false.
    [ ZnEasy get: webAddress ] on: ConnectionTimedOut do: [ :excepion | timeOut := true ].
    self assert: timeOut.! !

TBXBlog initialize!
